#!/usr/bin/env python
#--------------------------------------------------------------------------------------------------------------
# Copyright (c) International Business Machines Corp., 2008
#
# IBM XIV Storage System Software
#
# Licensed materials provided under the terms of the IBM International Program License Agreement.
# See the software licensing materials that came with the IBM XIV Storage System for terms and conditions.
#
#--------------------------------------------------------------------------------------------------------------

from host_attach import logger, is_this_ip_address, execute_and_log, is_error_free
from platform import system
from socket import socket, AF_INET, SOCK_STREAM
from sys import exit  

def extract_ip_from_ipandport(text_line, splitter=':'):
	return text_line.split(splitter)[0]

def is_server_listening(ip='', port=5):
	if not is_this_ip_address(ip):
		return 0
	connection = socket(AF_INET, SOCK_STREAM)
	connection.settimeout(3)
	try:
		connection.connect((ip, port))
	except socket.timeout:
		return 0
	return 1
		
def is_2810_listening_to_3260(ip_address):
	return is_server_listening(ip_address, 3260)

def nextra_verify(nextra_ip_address):
	if not is_this_ip_address (nextra_ip_address):
		return 0
	if not is_2810_listening_to_3260(nextra_ip_address):
		return 0
	return 1

def connect_to_list_of_2810s(ip_addresses):
	for ip in ip_addresses:
		if not connect_to_2810(ip):
			return 0
	return 1

def disconnect_from_list_of_2810s(ip_addresses):
	for ip in ip_addresses:
		if not disconnect_from_2810(ip):
			return 0
	return 1	

def initiator_exists():
	initiator_name = execute_and_log('cat /etc/initiatorname.iscsi')
	if not is_error_free(initiator_name):
		return 0
	if not initiator_name.__contains__('InitiatorName'):
		return 0
	return 1

def create_initiator():
	output = execute_and_log('echo InitiatorName=`iscsi-iname` > /etc/initiatorname.iscsi')
	if is_error_free(output):
		return 1
	return 0

def already_discovered_2810(nextra_ip_address):
	output = execute_and_log('cat /etc/iscsi.conf')
	if not is_error_free(output):
		return 0
	if output.__contains__('DiscoveryAddress=' + nextra_ip_address):
		return 1
	return 0

def get_discovered_addresses():
	output = execute_and_log('iscsi-ls')
	if not is_error_free(output):
		return []
	discovered_addresses = []
	for line in output.splitlines():
		if lines.__contains__('TARGET ADDRESS'):
			ip = line.split(':')[1].strip()
			discovered_addresses.append(ip)
	return discovered_addresses

def make_discovery(nextra_ip_address):
	output = execute_and_log('echo DiscoveryAddress=' + nextra_ip_address + ' >> /etc/iscsi.conf')
	output = execute_and_log('/etc/init.d/iscsi restart')
	if not is_error_free(output):
		return 0
	return 1

def get_targets():
	output = execute_and_log('iscsi-ls')
	if not is_error_free(output):
		return 0
	targets = {}
	for line in output.splitlines():
		if line.__contains__('TARGET NAME'):
			target_name = line.split(':')[1].strip()
		if line.__contains__('TARGET ADDRESS'):
			target_ip = line.split(':')[1].strip()
		targets[target_ip] = target_name
	return targets

def iscsi_service_start():
	output = execute_and_log('/etc/init.d/iscsi start')
	if not is_error_free(output):
		return 0
	return 1

def iscsi_service_healthcheck():
#	output = execute_and_log('/etc/init.d/iscsi status')
#	if not is_error_free(output):
#	 	logger.error('iscsi daemon does not exists')
#		return 0
#	if not output.__contains__('running'):
#		logger.warning('iscsi daemon is not in running state')
	output = execute_and_log('chkconfig --list iscsi')
	if not is_error_free(output):
		logger.error('iscsi daemon is not configured properly')
		return 0
	if not output.__contains__('3:on'):
		logger.error('iscsi daemon is not configured properly')
		return 0		
	return 1

def remove_2810_discovery_entry(nextra_ip_address):
	output = execute_and_log('cat /etc/iscsi.conf | grep -v ' + nextra_ip_address)
	output = execute_and_log('/etc/init.d/iscsi restart')
	return 1

def get_connected_sessions():
	output = execute_and_log('iscsi-ls')
	if not is_error_free(output):
		return {}
	sessions = {}
	for line in output.splitlines():
		if line.__contains__('TARGET NAME'):
			target_name = line.split(':')[1].strip()
		if line.__contains__('TARGET ADDRESS'):
			target_ip = line.split(':')[1].strip()
		if line.__contains__('SESSION STATUS') and line.__contains__('ESTABLISHED'):
			sessions[target_ip] = target_name
	return sessions
	
def verify_iscsi_connections():
	targets = get_targets()
	sessions = get_connected_sessions()
	for ip in targets:
		if not targets[ip] == sessions[ip]:
			return 0
	return 1

def iscsi_scan():
	output = execute_and_log('iscsi-rescan')
	if not is_error_free(output):
		logger.debug('failed to rescan iscsi')
		return 0
	return 1

def iscsi_service_config():
	output = execute_and_log('chkconfig --add iscsi')
	output = execute_and_log('chkconfig --level 345 iscsi on')
	return 1

def iscsi_configure():
	if not initiator_exists():
		logger.info('creating iscsi initiator')
		if not create_initiator():
			logger.error('failed to create initiator')
			return 0
	logger.info('configuring iscsi daemon')
	if not iscsi_service_config():
		logger.error('failed to configure iscsi daemon')
		return 0
#	logger.info('starting iscsi daemon')
#	if not iscsi_service_start():
#		logger.error('failed to start iscsi daemon')
#		return 0
#	logger.info('iscsi is now configured')
	return 1		

def iscsi_verify():
	if not initiator_exists():
		if not create_initiator():
		  return 0
	if not iscsi_service_healthcheck():
		return 0
	return 1

def connect_to_2810(nextra_ip_address):
	if not nextra_verify(nextra_ip_address):
		logger.error(nextra_ip_address + ' is not an ip address')
		return 0
	logger.info('writing the new configuration file and restarting the iscsi daemon')
	if not make_discovery(nextra_ip_address):
		logger.error('failed to discover targets from ' + nextra_ip_address)
		return 0
	return 1

def disconnect_from_2810(nextra_ip_address):
	if not nextra_verify(nextra_ip_address):
		return 0
	logger.info('writing the new configuration file and restrating the iscsi daemon')
	remove_2810_discovery_entry(nextra_ip_address)
	return 1

def iscsi_print_initiator():
	output = execute_and_log('cat /etc/initiatorname.iscsi | grep InitiatorName')
	if not is_error_free(output):
		logger.error('/etc/initiatorname.iscsi not found')
		return 0
	if not output.__contains__('InitiatorName='):
		logger.error('/etc/initiatorname.iscsi does not contain an iqn, run this script with -c')
		return 0
	print output
	return 1

